 /*
 * MAIN Generated Driver File
 * 
 * @file main.c
 * 
 * @defgroup main MAIN
 * 
 * @brief This is the generated driver implementation file for the MAIN driver.
 *
 * @version MAIN Driver Version 1.0.0
*/

/*
 [2023] Microchip Technology Inc. and its subsidiaries.

    Subject to your compliance with these terms, you may use Microchip 
    software and any derivatives exclusively with Microchip products. 
    You are responsible for complying with 3rd party license terms  
    applicable to your use of 3rd party software (including open source  
    software) that may accompany Microchip software. SOFTWARE IS ?AS IS.? 
    NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS 
    SOFTWARE, INCLUDING ANY IMPLIED WARRANTIES OF NON-INFRINGEMENT,  
    MERCHANTABILITY, OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT 
    WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY 
    KIND WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF 
    MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE 
    FORESEEABLE. TO THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP?S 
    TOTAL LIABILITY ON ALL CLAIMS RELATED TO THE SOFTWARE WILL NOT 
    EXCEED AMOUNT OF FEES, IF ANY, YOU PAID DIRECTLY TO MICROCHIP FOR 
    THIS SOFTWARE.
*/
#include "mcc_generated_files/system/system.h"
#include "app.h"
#include "../../../project_config.h"
#include <string.h>
#include "hal_gpio_harmony.h"
#include <xc.h>

/*
    Main application
*/

RESPONSE send_challenge_to_accessory(uint8_t *challenge);
RESPONSE verify_accessory_response(uint8_t chal_num);

void atCRC(size_t length, const uint8_t *data, uint8_t *crc_le);
void printHexArray(const unsigned char* array, size_t size, size_t bytesPerLine);

APP_STATE current_state;
RESPONSE status;

uint8_t packet[PACKET_SIZE+1];
uint8_t response[RESPONSE_SIZE];
uint8_t ctr[7];

extern uint8_t challenges[5][32];
extern uint8_t responses[5][32];
uint8_t chal_num;

int main(void)
{
    uint8_t *challenge;
    chal_num = 0;
    while(1)
    {
        switch(current_state)
        {
            case CR_INIT_APP:
            {
                SYSTEM_Initialize();
                current_state = CR_SEND_CHALLENGE;
                break;
            }
            case CR_SEND_CHALLENGE:
            {
                challenge = challenges[chal_num];
                printf("****** Sending Challenge to Accessory Device ******\r\n\n");
                printf("Challenge being sent to accessory:\r\n");
                printHexArray(challenge, 32, 8);
                status = send_challenge_to_accessory(challenge);
                if(status == ATCA_SUCCESS)
                {
                    current_state = CR_VERIFY_RESPONSE;
                }
                else
                {
                    current_state = CR_DONE;
                }
                break;
            }
            case CR_VERIFY_RESPONSE:
            {
                printf("****** Verifying Challenge Response ******\r\n\n");
                printf("Expected accessory response:\r\n");
                printHexArray(responses[chal_num], 32, 8);
                status = verify_accessory_response(chal_num);
                current_state = CR_DONE;
                break;
            }
            case CR_DONE:
            {
                if (status != ATCA_SUCCESS)
                {
                    printf("Challenge Failed by Accessory Device\r\n");
                }
                else
                {
                    printf("Accessory Authenticated Successfully!\r\n");
                }
                while(1);
            }
        }
        
    }    
}

RESPONSE send_challenge_to_accessory(uint8_t *challenge)
{
    //Create the packet
    packet[0] = 0x03;
    packet[1] = 0x27;
    packet[2] = MAC_OPCODE;
    packet[3] = MAC_MODE;
    packet[4] = SYMM_SLOT;
    packet[5] = 0;
    
    memcpy(&packet[6], challenge, 32);
    //Compute CRC and add to end of packet
    uint8_t crc[2];
    atCRC(37, &packet[1], crc);
    memcpy(&packet[38], crc, 2);
    
#if SELECTED_SHA104_SWI
    uint16_t txlength, rxlength;
    txlength = sizeof(packet);
    rxlength = RESPONSE_SIZE;
    hal_gpio_wake(SWI_ADDR);
    hal_gpio_send(SWI_ADDR, packet, &txlength);
    __delay_ms(15);
    hal_gpio_receive(SWI_ADDR, response, &rxlength);    
#else
    //Transmit packet to SHA104 over I2C and get response
    if (I2C1_Host_Write(I2C_ADDR, packet, PACKET_SIZE + 1))
    {
        while(I2C1_Host_IsBusy())
        {
            I2C1_Host_Tasks();
        }
        if (I2C1_Host_ErrorGet() == I2C_ERROR_NONE)
        {
            /* Write operation is successful */
        }
        else
        {
            /* Error handling */
        }
    }    
    // Give time for the SHA device to be 
    __delay_ms(16);
    if (I2C1_Host_Read(I2C_ADDR, response, RESPONSE_SIZE))
    {
        while(I2C1_Host_IsBusy())
        {
            I2C1_Host_Tasks();
        }
        if (I2C1_Host_ErrorGet() == I2C_ERROR_NONE)
        {
            /* Write operation is successful */
        }
        else
        {
            /* Error handling */
        }
    }            

#endif
    return status;
}

RESPONSE verify_accessory_response(uint8_t chal_num)
{
    uint8_t response_data[32];
    uint8_t response_crc[2];
    memcpy(response_data, &response[1], 32);
    atCRC(33, response, response_crc);
    
    printf("Actual accessory response:\r\n");
    printHexArray(response_data, 32, 8);
    
    if (response_crc[0] != response[33] || response_crc[1] != response[34])
        return ATCA_CHECKMAC_VERIFY_FAILED;
    for (uint8_t i = 0; i < 32; i++)
    {
        if (response_data[i] != responses[chal_num][i])
            return ATCA_CHECKMAC_VERIFY_FAILED;
    }
    return ATCA_SUCCESS;
}

void atCRC(size_t length, const uint8_t *data, uint8_t *crc_le)
{
    size_t counter;
    uint16_t crc_register = 0;
    uint16_t polynom = 0x8005;
    uint8_t shift_register;
    uint8_t data_bit, crc_bit;

    for (counter = 0; counter < length; counter++)
    {
        for (shift_register = 0x01; shift_register > 0x00; shift_register <<= 1)
        {
            data_bit = (data[counter] & shift_register) ? 1 : 0;
            crc_bit = crc_register >> 15;
            crc_register <<= 1;
            if (data_bit != crc_bit)
            {
                crc_register ^= polynom;
            }
        }
    }
    crc_le[0] = (uint8_t)(crc_register & 0x00FF);
    crc_le[1] = (uint8_t)(crc_register >> 8);
}

void printHexArray(const unsigned char* array, size_t size, size_t bytesPerLine) {
    for (size_t i = 0; i < size; i++) {
        printf("%02X ", array[i]);
        if ((i + 1) % bytesPerLine == 0) {
            printf("\r\n");
        }
    }
    printf("\r\n");
}
